/**
 *  Copyright (C) 2004-2005 Savoir-Faire Linux inc.
 *  Author: Yan Morin <yan.morin@savoirfairelinux.com> (cc++ mutex)
 *  Author: Jean-Philippe Barrette-LaPierre
 *             <jean-philippe.barrette-lapierre@savoirfairelinux.com>
 *                                                                              
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *                                                                              
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *                                                                              
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef SFLPHONEGUI_OBJECTPOOL_INL
#define SFLPHONEGUI_OBJECTPOOL_INL

#include <iostream>

template< typename T >
void
ObjectPool< T >::push(const T &value)
{
  ost::MutexLock guard(mMutex);
  mPool.push_back(value);
  mSemaphore.post();
}

template< typename T >
bool
ObjectPool< T >::pop(T &value, unsigned long time)
{
  //ost::MutexLock guard(mMutex);
  if (mSemaphore.wait(time)) {

    if(mPool.begin() == mPool.end()) {
      return false;
    } else {
      typename std::list< T >::iterator pos = mPool.begin();
      value = (*pos);
      mPool.pop_front();
      return true;
    }
  }
  return false;
}

#endif
