/**
 *  Copyright (C) 2005 Savoir-Faire Linux inc.
 *  Author: Yan Morin <yan.morin@savoirfairelinux.com>
 *                                                                              
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *                                                                              
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef REQUESTMANAGER_H
#define REQUESTMANAGER_H

#include <cc++/thread.h>

#include "sessionio.h"
#include "requestfactory.h"
#include "responsemessage.h"

/**
@author Yan Morin
*/
class RequestManager{
public:
    RequestManager();

    ~RequestManager();

    int exec(void);
    void sendResponse(const ResponseMessage& response);
    void quit() { _quit = true; }
    void stop() { _stop = true; quit(); }

private:
  void flushWaitingRequest();
  void handleExecutedRequest(Request * request, const ResponseMessage& response);

  RequestFactory _factory;
  SessionIO*  _sessionIO;
  ost::Mutex  _sessionMutex;

  // waiting requests
  ost::Mutex _waitingRequestsMutex;
  std::map<std::string, Request*> _waitingRequests;

  bool _quit; // tell to quit and end the exec loop for one client
  bool _stop; // tell to stop the exec loop
};

#endif
